/*****************************************************************************
    AmyEdit - A GTKMM LaTeX Editor
    Copyright (C) 2004-2005 Rob Page
    email:  page <dot> rob <at> gmail <dot> com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/

#ifdef USE_PRECOMPILED_HEADERS
#  include "all.hh"
#endif

#include "config.h"

/*
 * Standard gettext macros.
 */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (GETTEXT_PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif

#include "NewDialog.hh"
#include <gtkmm/button.h>
#include <gtkmm/label.h>
#include <gtkmm/stock.h>
#include <gtkmm/separator.h>
#include <gtkmm/alignment.h>
#include <gtkmm/entry.h>
#include <gtkmm/messagedialog.h>
#include <fstream>

class ListCols : public Gtk::TreeModel::ColumnRecord
{
public:

  ListCols()
  { add(col); }
  
  Gtk::TreeModelColumn<Glib::ustring> col;
}column;


NewDialog::NewDialog():
  a4paper("a4paper"),
  a5paper("a5paper"),
  article("article"),
  report("report"),
  parindent("Indent Paragraphs"),
  doublespacing("Double Spacing")
{  
  /* Standard Tab */

  list =Gtk::ListStore::create(column);
  templates.set_model(list);

  
  Glib::Dir dir (Glib::get_home_dir()+"/.amyedit/templates");
  Glib::ustring word;

  while ((word=dir.read_name())  != "")
    {
      Gtk::TreeModel::Row row = *(list->append());
      row[column.col] = word;
      
    }
	
  	
  templates.append_column("Templates", column.col);
  
  Gtk::TreeModel::iterator iter = list->children().begin();
  if(iter)
    templates.get_selection()->select(iter);
  
  Gtk::VBox *box = Gtk::manage(new class Gtk::VBox());
  box->add(templates);

  tabs.append_page(*box, "Standard");


  /* Advanced Tab */

   Gtk::RadioButton::Group paper_group;
   Gtk::RadioButton::Group doctype_group;

   a4paper.set_group(paper_group);
   a5paper.set_group(paper_group);

   article.set_group(doctype_group);
   report.set_group(doctype_group);

   parindent.set_active();

   Gtk::VBox *advbox = Gtk::manage(new class Gtk::VBox());
   Gtk::Button *newbutton = Gtk::manage(new class Gtk::Button("Save Template"));
   
   advbox->add(*Gtk::manage(new class Gtk::Label(_("Paper Type"))));
   advbox->add(a4paper);
   advbox->add(a5paper);
   advbox->add(*Gtk::manage(new class Gtk::HSeparator()));
   advbox->add(*Gtk::manage(new class Gtk::Label(_("Document Type"))));
   advbox->add(article);
   advbox->add(report);
   advbox->add(*Gtk::manage(new class Gtk::HSeparator()));
   advbox->add(*Gtk::manage(new class Gtk::Label(_("Misc Settings"))));
   advbox->add(parindent);
   advbox->add(doublespacing);
   advbox->add(*Gtk::manage(new class Gtk::HSeparator()));
   advbox->add(*newbutton);
   
   newbutton->signal_clicked().connect(sigc::mem_fun(*this, &NewDialog::on_save_template));
 
   tabs.append_page(*advbox, "Advanced");

   get_vbox()->add(tabs);
   add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
   add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
   
   set_title(_("New Document"));
   show_all_children();
}

NewDialog::~NewDialog()
{ 

}

Glib::ustring NewDialog::getDoc()
{
  Glib::ustring tabtext=tabs.get_tab_label_text(*tabs.get_nth_page(tabs.get_current_page()));
  if (tabtext=="Standard")
    {
      Gtk::TreeModel::iterator iter = templates.get_selection()->get_selected();
      if (iter)
	{
	  Gtk::TreeModel::Row row = *iter;
	  Glib::ustring name = row[column.col];
	  return Glib::file_get_contents(Glib::get_home_dir()+"/.amyedit/templates/"+name);
	}
      else 
	return Glib::ustring("");
    }
  else 
    {
      return getDocString();
    }
}

Glib::ustring NewDialog::getDocString()
{
  Glib::ustring docString="";
  
  docString+= "\\documentclass[";
  
  if (a4paper.get_active())
    docString+="a4paper";
  else if (a5paper.get_active())
    docString+="a5paper";

  docString+= "]{";

  if (article.get_active())
    docString +="article";
  else if (report.get_active())
    docString +="report";
  
  docString+= "}\n";

  if (!parindent.get_active() || doublespacing.get_active())
    docString +="\\usepackage{setspace}\n";

  if(!parindent.get_active())
    docString +="\\setlength{\\parindent}{0pt}\n\\setlength{\\parskip}{\\baselineskip}\n";

  if(doublespacing.get_active())
    docString +="\\doublespacing{}\n";
  
  docString +="\\begin{document}\n";

  docString +="\n\n\n";

  docString +="\\end{document}\n";

  return docString;

}

void NewDialog::on_save_template()
{
  Gtk::Dialog savedialog;
  // Create a dialog with a text entry, to save template as..
  savedialog.set_title("Save Template");
  Gtk::VBox *vbox=savedialog.get_vbox();
  Gtk::Entry templatename;
  
  vbox->add(*Gtk::manage(new class Gtk::Label("Enter Name:")));
  vbox->add(templatename);

  savedialog.add_button(Gtk::Stock::OK,Gtk::RESPONSE_OK);
  savedialog.add_button(Gtk::Stock::CANCEL,Gtk::RESPONSE_CANCEL);

  savedialog.show_all_children();
  if (savedialog.run() == Gtk::RESPONSE_OK)
    {
      std::ofstream outfile((Glib::ustring(getenv("HOME")) + "/.amyedit/templates/"+templatename.get_text()).c_str());
      if (!outfile)
	{
	  Gtk::MessageDialog dialog("Cannot open ~/.amyedit/templates/"+templatename.get_text()+" to save configuration\nThis could be a permissions error");
	  dialog.run();
	}
      else
	{
	  outfile << getDocString();
	}
      outfile.close();
    }

}
