/*****************************************************************************
    AmyEdit - A GTKMM LaTeX Editor
    Copyright (C) 2004-2005 Rob Page
    email:  page <dot> rob <at> gmail <dot> com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/

#ifdef USE_PRECOMPILED_HEADERS
#  include "all.hh"
#endif

#include "config.h"

/*
 * Standard gettext macros.
 */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (GETTEXT_PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif

#include "Preferences.hh"

#include <gtkmm/stock.h>
#include <gtkmm/notebook.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/table.h>
#include <gtkmm/messagedialog.h>
#include <gtkmm/spinbutton.h>
#include <gtkmm/adjustment.h>

#include <fstream>
#include <map>

PrefStruct::PrefStruct()
{

  // Set Default Preferences
  tab_size=4;
  word_wrap_style="Coming_Soon!";
  latex_command="pdflatex temp.tex";
  preview_command="xpdf temp.pdf";
  window_height=400;
  window_width=600;
  output_file="temp.pdf";
  language="en_GB";
  syntax_highlighting=true;
  recent_files[0]="";
  recent_files[1]="";
  recent_files[2]="";
  recent_files[3]="";
  recent_files[4]="";

  std::ifstream infile((Glib::get_home_dir() + "/.amyedit/amyconf").c_str());

  if (infile)
    {
      std::map<Glib::ustring, Glib::ustring *> prefs;
      prefs["word_wrap_style"] = &word_wrap_style;
      prefs["latex_command"] = &latex_command;
      prefs["preview_command"] = &preview_command;
      prefs["output_file"] = &output_file;
      prefs["language"] = &language;
      prefs["recent0"] = &recent_files[0];
      prefs["recent1"] = &recent_files[1];
      prefs["recent2"] = &recent_files[2];
      prefs["recent3"] = &recent_files[3];
      prefs["recent4"] = &recent_files[4];

      Glib::ustring syn_tmp = "";
      prefs["syntax_highlighting"] = &syn_tmp;

      Glib::ustring tab_size_tmp = "";
      prefs["tab_size"] = &tab_size_tmp;

      Glib::ustring win_h_tmp = "";
      prefs["window_height"] = &win_h_tmp;

      Glib::ustring win_w_tmp = "";
      prefs["window_width"] = &win_w_tmp;

      Glib::ustring var1,var2;

      infile >> var1 >> var2 ;
      std::string var3;
      
      while (!infile.eof())
	{
	  infile.ignore();
	  getline(infile, var3);

	  *(prefs[var1])=Glib::ustring(var3);
	  infile >> var1 >> var2 ;
	}

      if (syn_tmp == "off")
	syntax_highlighting=false;

      if (tab_size_tmp != "")
	tab_size = atoi(tab_size_tmp.c_str());

      if (win_h_tmp != "")
	window_height = atoi(win_h_tmp.c_str());
      
      if (win_w_tmp != "")
	window_width = atoi(win_w_tmp.c_str());

    }
  infile.close();

}

PrefStruct::~PrefStruct()
{
  std::ofstream outfile((Glib::get_home_dir() + "/.amyedit/amyconf").c_str());
  if (!outfile)
    {
      Gtk::MessageDialog dialog("Cannot open ~/.amyedit/amyconf to save configuration\nThis could be a permissions error");
      dialog.run();
    }
  else
    {
      outfile << "tab_size = " << tab_size << std::endl;
      outfile << "word_wrap_style = " << word_wrap_style << std::endl;
      outfile << "latex_command = " << latex_command << std::endl;
      outfile << "preview_command = " << preview_command << std::endl;
      outfile << "output_file = " << output_file << std::endl;
      outfile << "window_height = " << window_height << std::endl;
      outfile << "window_width = " << window_width << std::endl;
      outfile << "language = " << language << std::endl;
      outfile << "recent0 = " << recent_files[0] << std::endl;
      outfile << "recent1 = " << recent_files[1] << std::endl;
      outfile << "recent2 = " << recent_files[2] << std::endl;
      outfile << "recent3 = " << recent_files[3] << std::endl;
      outfile << "recent4 = " << recent_files[4] << std::endl;
      outfile << "syntax_highlighting = ";
      if (syntax_highlighting == true)
	outfile << "on";
      else
	outfile << "off";
      outfile << std::endl;
    }

  outfile.close();
}

PrefStruct::PrefStruct(PrefStruct *me)
{

}

Preferences::Preferences(PrefStruct myPrefs) :
  highlightbutton("Syntax Highlighting")
{
  curPrefs=myPrefs;

  set_title("AmyEdit Preferences");
  
  Gtk::Notebook *preftabs=Gtk::manage(new class Gtk::Notebook());
  

  /***********************************
   * First Preferences Tab : Editing *
   ***********************************/

  Gtk::Table *tab1=Gtk::manage(new class Gtk::Table(2,2));

  tab1->attach(*Gtk::manage(new class Gtk::Label(_("Tab Size"))),0,1,1,2);
  Gtk::Adjustment* adj = Gtk::manage(new class Gtk::Adjustment(myPrefs.tab_size,1,128));
  entries1[0] = Gtk::manage(new class Gtk::SpinButton(*adj,1.0,0));
  tab1->attach(*entries1[0],1,2,1,2);

  tab1->attach(*Gtk::manage(new class Gtk::Label(_("Spellcheck Language"))),0,1,2,3);
  entries1[2]=Gtk::manage(new class Gtk::Entry());
  entries1[2]->set_text(myPrefs.language);
  tab1->attach(*entries1[2],1,2,2,3);

  highlightbutton.set_active(myPrefs.syntax_highlighting);
  tab1->attach(highlightbutton,0,2,3,4);


  /*************************************
   * Second Preferences Tab : Commands *
   *************************************/

  Gtk::Table *tab2=Gtk::manage(new class Gtk::Table(2,4));

  tab2->attach(*Gtk::manage(new class Gtk::Label(_("LaTeX Command:"))),0,1,0,1);
  entries2[0]=Gtk::manage(new class Gtk::Entry());
  entries2[0]->set_text(myPrefs.latex_command);
  tab2->attach(*entries2[0],1,2,0,1);

  tab2->attach(*Gtk::manage(new class Gtk::Label(_("File to operate on will be called 'temp.tex'"))),0,2,1,2);
		    
  tab2->attach(*Gtk::manage(new class Gtk::Label(_("Output File Name:"))), 0,1,2,3);
  entries2[1]=Gtk::manage(new class Gtk::Entry());
  entries2[1]->set_text(myPrefs.output_file);
  tab2->attach(*entries2[1],1,2,2,3);

  tab2->attach(*Gtk::manage(new class Gtk::Label(_("Preview Command:"))),0,1,3,4);
  entries2[2]=Gtk::manage(new class Gtk::Entry());
  entries2[2]->set_text(myPrefs.preview_command);
  tab2->attach(*entries2[2],1,2,3,4);

  
  /*************************************
   * Setting Up The Rest of the Dialog *
   *************************************/

  preftabs->append_page(*tab1,"General");
  preftabs->append_page(*tab2,"Commands");

  get_vbox()->add(*preftabs);
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
 
  show_all_children();

}

Preferences::~Preferences()
{
}


int Preferences::run()
{
  int result=Gtk::Dialog::run();
   
  curPrefs.tab_size=(int)dynamic_cast<Gtk::SpinButton*>(entries1[0])->get_value();
  //curPrefs.word_wrap_style=dynamic_cast<Gtk::Entry*>(&widgets1[1][1]).get_text();
  curPrefs.language=entries1[2]->get_text();

  curPrefs.latex_command=entries2[0]->get_text();
  curPrefs.output_file=entries2[1]->get_text();
  curPrefs.preview_command=entries2[2]->get_text();
  curPrefs.syntax_highlighting=highlightbutton.get_active();
  
  return result;

}
